%% PURPOSE :
%% Group tracking by means of Convolution Particle Filtering
%% rectangular approximation of the group size
%% group center state = [x_pos, x_speed, y_pos, y_speed]'
%% extent parameters = [a, b]';
clear all
close all

global dt_s N nx nz nv R_sqrt clut clutterRo ones_N

%*******************************************************************
% INITIAL CONDITIONS:
% ==============================
clut = 1; % 0 - no clutter. 1 - clutter added.
clutterRo = 1e-2;

%------ sensor data
%
nz   =  2;              % dimension of the measurement vector (x, y)
nv   =  2;              % dimension of the measurement noise vector
std_x = 0.1;            %[m] measurement error standard deviations
std_y = 0.1;            %[m]

% measurement error covariance matrix ;
R_sqrt  =  [ std_x   0 ; 0   std_y];
R  =       [ std_x^2 0 ; 0   std_y^2];

% ------ dynamic model data
%
nx   =  6;         % dimension of the filter state vector
ne   =  2;         % dimension of the extent parameters vector

state = [100,0,100,0,40,40]; % initial state
t_max_s = 40; % simulation time
dt_s = 0.125; % sampling time
kmax = ceil(t_max_s/dt_s); % maximum number of time steps (final time step)
N = 1000;              % Number of particles
MC_runs = 1;           % number of Monte Carlo runs

x_est_all    = zeros(MC_runs,nx-ne,kmax);
tita_est_all = zeros(MC_runs,ne,kmax);

%----- initial state
%
std_x0 =[50,10,50,10,30,30]';

%
% ----- CPF constant parameters
%
ones_N = ones(1,N);

if MC_runs > 1
    try
        load MCRuns
        start = length(time_pf)+1;
    catch
        start = 1;
    end
else
    start = 1;
end
%  ===== Begin the Monte Carlo runs =====
for m = start:MC_runs
    
    rand('state',sum(500*clock));    % Shuffle the pack!
    randn('state',sum(800*clock));   % Shuffle the pack!   Bernoulli
    fprintf('run = %i / %i :  \r',m,MC_runs);
    
    crowdInitialise; % Generate ground truth data and measurements
    [grState.true_state] = state(1,1:4)';
    [grState.true_extent] = state(1,5:6)';
    x_0 = [grState.true_state;grState.true_extent];
    [xParticle_pf, w] = init_group_state(x_0,std_x0);
    tic;               % Initialise timer for benchmarking
    %
    % ///////////////// ========= Evolution in time =========//////////////////
    for k = 3:kmax
        %
        fprintf('run = %i / %i :  PF : k =  %i / %i  \r',m,MC_runs,k,kmax);
        %
        % ----- prediction step
        %
        [xParticle_pred, c, ...
            ]= PF_prediction(xParticle_pf);
        
        clust.measurements= measurements(k).m';
        [grState.true_state] = state(k,1:4)';
        [grState.true_extent] = state(k,5:6)';
        
        % ===== EVALUATE IMPORTANCE WEIGHTS =====
        [w]= CPF_likelihood(xParticle_pred,...
            c, clust.measurements, w);
        %  output estimate
        x_est_k = xParticle_pred * w';
        %  resampling
        [xParticle_pf,w]= resampling(xParticle_pred',...
            w');
        
        x_est_all(m,:,k)      = x_est_k(1:4);
        tita_est_all(m,:,k) = x_est_k(5:6);
        perf.est(k,:,m) = x_est_k;
        perf.state(k,:,m) = [grState.true_state;grState.true_extent];
    end
    
    time_pf(m) = toc;       % How long did this take?
    %
    save('MCRuns','perf','time_pf')
end;


mean_time_pf      = mean(time_pf);
%
% display final results
disp(' ');
disp('Mean Execution time (seconds)');
disp('-------------------------');
disp(' ');
disp(['PF           = ' num2str(mean_time_pf)]);
%************************************************************
plot_crowd
